'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});
exports.LibraryContributor = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _validation = require('./validation');

var _zlib = require('zlib');

var _zlib2 = _interopRequireDefault(_zlib);

var _tarFs = require('tar-fs');

var _tarFs2 = _interopRequireDefault(_tarFs);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _tmp = require('tmp');

var _tmp2 = _interopRequireDefault(_tmp);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var LibraryContributor = exports.LibraryContributor = function () {

	/**
  * @param {FileSystemLibraryRepository} repo  The repo containing the library.
  * @param {Particle.Client} client  The particle-api.js client.
  */
	function LibraryContributor(_ref) {
		var repo = _ref.repo,
		    client = _ref.client;

		_classCallCheck(this, LibraryContributor);

		Object.assign(this, { repo: repo, client: client });
	}

	/**
  * Creates a tar.gz stream containing the contents of the given directory in the file system that can be piped to another stream.
  * @param {string} dir The directory to tar.gz
  * @returns {ReadableStream} a stream that can be piped to a writableStream to provide the tar.gz file.
  */


	_createClass(LibraryContributor, [{
		key: 'targzdir',
		value: function targzdir(dir) {
			return new Promise(function (fulfill, reject) {
				// WORKAROUND: form-data in superagent in particle-api-js only support file streams so copy to a temporary file
				var archive = _tmp2.default.fileSync();

				var archiveWriter = _fs2.default.createWriteStream(archive.name);
				var gzip = _zlib2.default.createGzip();

				var pack = _tarFs2.default.pack(dir, {
					ignore: function ignore(name) {
						var dirs = _path2.default.dirname(_path2.default.relative(dir, name)).split(_path2.default.sep);
						return dirs.length && dirs[0] === '.git';
					}
				});
				pack.pipe(gzip).pipe(archiveWriter);

				archiveWriter.on('finish', function () {
					var archiveReader = _fs2.default.createReadStream(archive.name);
					fulfill(archiveReader);
				});

				archiveWriter.on('error', reject);
			});
		}
	}, {
		key: '_contribute',
		value: function _contribute(name, stream) {
			return this.client.contributeLibrary(stream);
		}
	}, {
		key: '_validateLibrary',
		value: function _validateLibrary(repo, name) {
			return (0, _validation.validateLibrary)(repo, name);
		}

		/**
   * Contributees a library with the given name from the repo.
   * @param {function} callback  Called during the contributing process:
   *  callback('validatingLibrary', name)
   *  callback('contributingLibrary', library)
   *  callback('contributeComplete', library)
   *
   * @param {string} name The name of the library to contribute.
   * @param {boolean} dryRun When true, the library is only validated, and not contributeed.
   * @return {Promise} to contribute the named library.
   */

	}, {
		key: 'contribute',
		value: function contribute(callback, name) {
			var dryRun = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : false;

			var libraryDirectory = this.repo.libraryDirectory(name);
			return this._doContribute(callback, name, libraryDirectory, dryRun);
		}
	}, {
		key: '_doContribute',
		value: function _doContribute(callback, name, libraryDirectory, dryRun) {
			var _this = this;

			var validatePromise = this._buildValidatePromise(name);
			// mdm - not sure about allowing the site to wrap the promise since it can potentially cause parts of the command
			// code to not execute if the returned promise doesn't chain the validatePromise.
			return this._buildNotifyPromise(callback, 'validatingLibrary', validatePromise, libraryDirectory).then(function () {
				return _this._doContributeDirect(callback, name, libraryDirectory, dryRun);
			});
		}
	}, {
		key: '_doContributeDirect',
		value: function _doContributeDirect(callback, name, libraryDirectory, dryRun) {
			var _this2 = this;

			return this.repo.fetch(name).then(function (library) {
				return _this2._doContributeLibrary(callback, library, libraryDirectory, dryRun);
			});
		}
	}, {
		key: '_doContributeLibrary',
		value: function _doContributeLibrary(callback, library, libraryDirectory, dryRun) {
			var contributePromise = this._buildContributePromise(libraryDirectory, library.name, dryRun);
			var notify = this._buildNotifyPromise(callback, 'contributingLibrary', contributePromise, library);
			return notify.then(function () {
				return callback('contributeComplete', library);
			});
		}

		/**
   * Notifies the callback function with the given notification and promise and optinoal arguments.
   * The callback may chain the promise or return a false value.
   * @param {function} callback      The callback to call
   * @param {string} notify        The notification event
   * @param {Promise} promise       The promise that will be executed
   * @param {Array} other         Additional arguments
   * @returns {*|Promise.<*>} The promise that will notify the callback and run the promise.
   * @private
   */

	}, {
		key: '_buildNotifyPromise',
		value: function _buildNotifyPromise(callback, notify, promise) {
			for (var _len = arguments.length, other = Array(_len > 3 ? _len - 3 : 0), _key = 3; _key < _len; _key++) {
				other[_key - 3] = arguments[_key];
			}

			return Promise.resolve().then(function () {
				return callback.apply(undefined, [notify, promise].concat(other));
			}).then(function (wrapped) {
				return wrapped || promise;
			});
		}
	}, {
		key: '_buildValidatePromise',
		value: function _buildValidatePromise(name) {
			return this._validateLibrary(this.repo, name).then(function (results) {
				if (results && !results.valid) {
					var error = new Error('Library is not valid. ' + (0, _validation.validationMessage)(results));
					error.validate = results;
					throw error;
				}
			});
		}

		/**
   * Constructs a promise to perform the contributing of the library.
   * @param {string} libraryDirectory      The directory of the library to contribute.
   * @param {string} libraryName           The name of the library in the repo
   * @param {boolean} dryRun               When true, the library is only zipped, and not contributeed.
   * @returns {*|Promise.<boolean>}       Promise to contribute the library.
   * @private
   */

	}, {
		key: '_buildContributePromise',
		value: function _buildContributePromise(libraryDirectory, libraryName, dryRun) {
			var _this3 = this;

			return Promise.resolve(this.targzdir(libraryDirectory)).then(function (pipe) {
				return dryRun ? true : _this3._contribute(libraryName, pipe);
			});
		}
	}]);

	return LibraryContributor;
}();